<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\App;


use Magento\Framework\App;
use Magento\Framework\App\Area;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\App\Request\Http as RequestHttp;
use Magento\Framework\App\Response\Http as ResponseHttp;
use Magento\Store\Model\ScopeInterface;

class Geoip implements \Magento\Framework\AppInterface
{
     /**
     * @var State
     */
    protected $_state;

    /**
     * @var Console\Request
     */
    protected $_request;

    /**
     * @var Console\Response
     */
    protected $_response;

    /**
     * Object manager
     *
     * @var ObjectManagerInterface
     */
    private $objectManager;

    /**
     * Inject dependencies
     *
     * @param State $state
     * @param Console\Request $request
     * @param Console\Response $response
     * @param ObjectManagerInterface $objectManager
     * @param array $parameters
     */
    public function __construct(
           
        \Magento\Framework\App\State $state,
        RequestHttp $request,
        ResponseHttp $response,
        ObjectManagerInterface $objectManager,
        array $parameters = []
    ) {
        
        $this->_state = $state;
        $this->_request = $request;
        $this->_request->setParams($parameters);
        $this->_response = $response;
        $this->objectManager = $objectManager;
    }

    /**
     * Run application
     *
     * @return ResponseInterface
     */
    public function launch()
    {
        $this->_state->setAreaCode(Area::AREA_FRONTEND);
        $configLoader = $this->objectManager->get('Magento\Framework\ObjectManager\ConfigLoaderInterface');
        $this->objectManager->configure($configLoader->load(Area::AREA_FRONTEND));

        $this->_helper = $this->objectManager->get('\FiloBlu\ExtAmastyGeoipRedirect\Helper\Data');
        $this->_logger = $this->objectManager->get('\FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger');
        $this->_multiDomainHelper = $this->objectManager->get('\FiloBlu\MultiDomain\Helper\Data');
        $this->remoteAddress = $this->objectManager->get('\Magento\Framework\HTTP\PhpEnvironment\RemoteAddress');
        $this->scopeConfig = $this->objectManager->get('\Magento\Framework\App\Config\ScopeConfigInterface');
        $this->geoipHelper = $this->objectManager->get('\Amasty\GeoipRedirect\Helper\Data');
        $this->urlBuilder = $this->objectManager->get('\Magento\Framework\UrlInterface');
        $this->storeManager = $this->objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $this->geolocation = $this->objectManager->get('\Amasty\Geoip\Model\Geolocation');
        $this->customerSession = $this->objectManager->get('\Magento\Customer\Model\Session');
        $this->storeCookieManager = $this->objectManager->get('\Magento\Store\Api\StoreCookieManagerInterface');
        
        $restrictionBypass = false;

        $request = $this->_request;

        if ($request->isXmlHttpRequest()) {
            return;
        }

        $response = $this->_response;

        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $logEnabled = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log', $scopeStores, $currentStoreId);
        $logEnabledForCookie = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_cookie', $scopeStores, $currentStoreId);
        $logEnabledForIps = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_ip', $scopeStores, $currentStoreId);

        $ipRestriction = $this->scopeConfig->getValue('amgeoipredirect/restriction/ip_restriction');
        $currentIp = $this->remoteAddress->getRemoteAddress();
        $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
        $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

        if ($logEnabled && $logEnabledForIps && !$currentIp) {
            $this->_logger->info("Can't get ip");
        }

        if ($logEnabled && $logEnabledForIps) {
            if (strpos($logEnabledForIps, $currentIp) != false || $logEnabledForIps == $currentIp) {
                
            } else {
                $logEnabled = false;
            }
        }

        if ($logEnabled && $logEnabledForCookie && !$this->_helper->getCanLogCookie()) {
            $logEnabled = false;
        }

        $this->redirectAllowed = $this->_helper->applyLogic($request, $logEnabled, $this->urlBuilder->getCurrentUrl(), $this->_logger);

        
        if ($logEnabled) {
            $this->_logger->info(" #### START REQUEST #### " . $this->urlBuilder->getCurrentUrl() . " controller->" . $request->getControllerModule());
        }


        if ($logEnabled) {
            $this->_logger->info("Current Ip: " . $currentIp);

            if ($fakeIp) {
                $this->_logger->info("Fake Ip: " . $fakeIp);
            }

            if ($fakeCountry) {
                $this->_logger->info("Fake Country: " . $fakeCountry);
            }
        }

        if ($fakeIp) {
            $currentIp = $fakeIp;
        }


        if (!empty($ipRestriction)) {
            $ipRestriction = array_map("rtrim", explode(PHP_EOL, $ipRestriction));
            foreach ($ipRestriction as $ip) {
                if ($currentIp && $currentIp == $ip) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Ip restriction " . $ip);
                    }
                    $restrictionBypass = true;
                }
            }
        }
        $isApi = $request->getControllerModule() == 'Mage_Api';
        if ($isApi || !$this->geoipHelper->isModuleOutputEnabled('Amasty_Geoip')) {
            if ($logEnabled) {
                $this->_logger->info("No Redirect For No Output enabled Module or API Call");
            }
            $restrictionBypass = true;
        }
        
        $userAgent = $request->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map("trim", $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Not Allowed User agent " . $userAgent);
                    }
                    $restrictionBypass = true;
                }
            }
        }


        if ($logEnabled) {
            if ($this->redirectAllowed) {
                $this->_logger->info("redirect is allowed");
            } else {
                $this->_logger->info("redirect is not allowed");
            }
        }

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed && !$this->_helper->notGeolocalizedRequest($request) && !$restrictionBypass) {

            $location = $this->geolocation->locate($currentIp);
            $country = $location->getCountry();

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            if ($logEnabled) {
                $this->_logger->info("Current Country: " . $country);
            }

            if ($country) {
                $this->_helper->setLocalizedCountryCookie($country);
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/restriction/first_visit_redirect')) {
                $getAmYetRedirectStore = $this->_helper->getStoreCookie();
                $getAmYetRedirectCurrency = $this->_helper->getCurrencyCookie();
                $getAmYetRedirectUrl = $this->_helper->getUrlCookie();
            } else {
                $getAmYetRedirectStore = 0;
                $getAmYetRedirectCurrency = 0;
                $getAmYetRedirectUrl = 0;
            }

            if (!$getAmYetRedirectUrl && $this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {
                $urlMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_url/url_mapping', $scopeStores, $currentStoreId));
                $currentUrl = $this->urlBuilder->getCurrentUrl();
                foreach ($urlMapping as $countries => $url) {
                    if (strpos($countries, $country) !== false && $url != $currentUrl) {
                        $this->_helper->setUrlCookie();
                        $url = preg_replace('/&?geolocation=[^&]*/', '', $url);
                        $response->setRedirect($url);
                        $response->sendResponse();
                        exit;
                    }
                }
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/country_store/enable_store')) {

                $redirect_always_from_slash = $this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_always_from_slash');
                $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));

                if ($getAmYetRedirectStore) {

                    $can_redirect = true;

                    if ($getAmYetRedirectStore == $this->storeManager->getStore()->getId() && !$this->_helper->geolocationRequest($request)) {
                        $can_redirect = false;
                    }
                    if ($redirect_always_from_slash && ($currentPath == "/" || $currentPath == "/geoip.php")) {
                        $can_redirect = true;
                    }

                    if ($can_redirect) {

                        $currentStoreUrl = str_replace('&amp;', '&', $this->storeManager->getStore($getAmYetRedirectStore)->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl);

                        if ($logEnabled) {
                            $this->_logger->info("Cookie is set, redirect to ->" . $redirectStoreUrl);
                        }

                        $this->storeCookieManager->setStoreCookie($this->storeManager->getStore($getAmYetRedirectStore));
                        $redirectStoreUrl = preg_replace('/&?geolocation=[^&]*/', '', $redirectStoreUrl);
                        $response->setRedirect($redirectStoreUrl);
                        $response->sendResponse();
                        exit;
                    } else {
                        if ($logEnabled) {
                            $this->_logger->info("Cookie is set, store is correct , no redirect required");
                        }
                    }
                } else {


                    if ($country) {
                        $this->_helper->setCountryCookie($country);

                        if ($logEnabled) {
                            $this->_logger->info("Set country cookie->" . $country);
                        }
                    } else {
                        $countryid = $this->scopeConfig->getValue('general/country/default', $scopeStores);
                        $this->_helper->setCountryCookie($countryid);
                        if ($logEnabled) {
                            $this->_logger->info("Set default country cookie->" . $country);
                        }
                    }

                    $allStores = $this->storeManager->getStores();

                    $currentGroup = $this->_multiDomainHelper->getCurrentGroup();

                    foreach ($allStores as $store) {

                        if (!$store->isActive() || ($currentGroup && $currentGroup != $this->_multiDomainHelper->getCurrentGroup($store->getWebsite()->getCode()))) {
                            continue;
                        }

                        $currentStoreUrl = str_replace('&amp;', '&', $store->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl);
                        $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());
                        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_between_websites')) {
                            $useMultistores = $store->getWebsiteId() == $this->storeManager->getStore()->getWebsiteId();
                        } else {
                            $useMultistores = true;
                        }

                        $can_redirect = true;

                        if ($store->getId() == $currentStoreId && !$this->_helper->geolocationRequest($request)) {
                            $can_redirect = false;
                        }

                        if ($redirect_always_from_slash && $currentPath == "/") {
                            $can_redirect = true;
                        }

                        if ($country && $countries && strpos($countries, $country) !== false && $can_redirect && $useMultistores
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Geolocalized, redirect to " . $redirectStoreUrl);
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            $redirectStoreUrl = preg_replace('/&?geolocation=[^&]*/', '', $redirectStoreUrl);
                            $response->setRedirect($redirectStoreUrl);
                            $response->sendResponse();
                            exit;
                        }

                        if ($country && $countries && strpos($countries, $country) !== false && $store->getId() == $currentStoreId
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Already in the right website, set only cookie no redirect necessary");
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            break;
                        }
                    }
                }
            }

            if (!$getAmYetRedirectCurrency && $this->scopeConfig->getValue('amgeoipredirect/country_currency/enable_currency')) {
                $currencyMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_currency/currency_mapping', $scopeStores, $currentStoreId));
                foreach ($currencyMapping as $countries => $currency) {
                    if (strpos($countries, $country) !== false && $this->storeManager->getStore()->getCurrentCurrencyCode() != $currency) {
                        $this->_helper->setCurrencyCookie();
                        $this->storeManager->getStore()->setCurrentCurrencyCode($currency);
                    }
                }
            }
        }

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed && !$this->_helper->notGeolocalizedRequest($request) && !$this->_helper->geolocationRequest($request)) {
            $currentUrl = $this->urlBuilder->getCurrentUrl();
            $currentUrl = preg_replace('/geoip.php/', '', $currentUrl);
            $currentUrl .= (parse_url($currentUrl, PHP_URL_QUERY) ? '&' : '?') . 'notgeolocalized=1';
            $response->setRedirect($currentUrl);
            $response->sendResponse();
            exit;
        }

        if ($logEnabled) {
            $this->_logger->info(" #### END REQUEST #### ");
        }
        
        return $response;
        
    }

    /**
     * {@inheritdoc}
     */
    public function catchException(App\Bootstrap $bootstrap, \Exception $exception)
    {
        return false;
    }
}
